/*
 * SPDX license identifier: MPL-2.0
 *
 * Copyright (C) 2011-2015, BMW AG
 *
 * This file is part of COVESA Project DLT - Diagnostic Log and Trace.
 *
 * This Source Code Form is subject to the terms of the
 * Mozilla Public License (MPL), v. 2.0.
 * If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * For further information see http://www.covesa.org/.
 */

/* generated file, do not edit */

#ifndef DLT_ID_H
#define DLT_ID_H

#define DLT_EXA3_CON_EXA3_ID1 1000
#define DLT_EXA3_CON_EXA3_ID2 1001
#define DLT_EXA3_CON_EXA3_ID3 1002

#endif /* DLT_ID_H */
